import json
import math
from datetime import datetime, timedelta
from operator import attrgetter

# start date: 2023-04-16T22:18:10.830Z
# https://stacker.news/items/165652
START_DAY = datetime.strptime("2023-04-16", '%Y-%m-%d')
TODAY = datetime.now()
# +1 since we start counting from 1
PASSED_DAYS = (TODAY - START_DAY).days + 1


def load_invoices(path):
    with open(path, 'r') as file:
        data = json.load(file)
        invoices = data["data"]["walletHistory"]["facts"]
    return invoices

class Participant:
    def __init__(self, name, day, paid=True):
        self.name = name
        self.day = day
        self.paid = paid
        self.eliminated = None
        self.eliminated_in = -1
        self.elimination_date = None
    def __str__(self):
        return f"| @{self.name} | {self.day} | {'X' if self.paid else ''} | {self.elimination_date if self.elimination_date else ''} | {'X' if self.eliminated else ''} |"

def parse_invoices(invoices):
    participants = [
        Participant("orthwyrm", 260),
        Participant("grayruby", 655),
        Participant("nikotsla", 570),
        Participant("siggy47", 380),
        Participant("Alby", 365),
        Participant("phatom", 440, False),
        Participant("Longtermwizard", 700),
        Participant("BitcoinIsTheFuture", 584),
        Participant("Bitman", 810),
        # not bad format but I want to use lowercase nym like in SN
        Participant("carlosfandango", 501),
    ]

    for i in invoices:
        status = i["status"]
        if status != "CONFIRMED":
            continue
        sats = i["sats"]
        if sats != 1000:
            continue
        comment = i["invoiceComment"].strip()
        parts = [s.strip() for s in comment.split("-")]
        if len(parts) != 3:
            # print("invalid format:", comment)
            continue
        _, name, day = parts
        # check if participant was not already added via hardcoded list
        found = False
        for p in participants:
            if p.name.lower() == name.lower().strip("@"):
                found = True
                break
        if not found:
            participants.append(Participant(name, int(day)))

    participants = list(sorted(participants, key=attrgetter('day')))
    return participants

def populate_eliminated(participants):
    if len(participants) < 2:
        return participants

    len_p = len(participants)
    for index, p1 in enumerate(participants):
        # last participant
        if index == len_p - 1:
            break
        p2 = participants[index+1]
        p1p2_center = p1.day + math.floor(abs(p1.day - p2.day)/2)
        p1.eliminated_in = p1p2_center - PASSED_DAYS
        p1.elimination_date = (TODAY + timedelta(days=p1.eliminated_in)).strftime("%Y-%m-%d")
        p1.eliminated = PASSED_DAYS >= p1p2_center

    return participants